-- Copyright (c) 2016-present, SoundCloud Ltd.
-- All rights reserved.
--
-- This source code is distributed under the terms of a MIT license,
-- found in the LICENSE file.

{-# LANGUAGE DeriveGeneric              #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# LANGUAGE TemplateHaskell            #-}

module Kubernetes.Model.V1.EmptyDirVolumeSource
    ( EmptyDirVolumeSource (..)
    , medium
    , mkEmptyDirVolumeSource
    ) where

import           Control.Lens.TH           (makeLenses)
import           Data.Aeson.TH             (defaultOptions, deriveJSON,
                                            fieldLabelModifier)
import           Data.Text                 (Text)
import           GHC.Generics              (Generic)
import           Prelude                   hiding (drop, error, max, min)
import qualified Prelude                   as P
import           Test.QuickCheck           (Arbitrary, arbitrary)
import           Test.QuickCheck.Instances ()

-- | Represents an empty directory for a pod. Empty directory volumes support ownership management and SELinux relabeling.
data EmptyDirVolumeSource = EmptyDirVolumeSource
    { _medium :: !(Maybe Text)
    } deriving (Show, Eq, Generic)

makeLenses ''EmptyDirVolumeSource

$(deriveJSON defaultOptions{fieldLabelModifier = (\n -> if n == "_type_" then "type" else P.drop 1 n)} ''EmptyDirVolumeSource)

instance Arbitrary EmptyDirVolumeSource where
    arbitrary = EmptyDirVolumeSource <$> arbitrary

-- | Use this method to build a EmptyDirVolumeSource
mkEmptyDirVolumeSource :: EmptyDirVolumeSource
mkEmptyDirVolumeSource = EmptyDirVolumeSource Nothing
